/*
 * $Id: timer.c,v 1.3 93/06/21 13:39:37 jopi Exp $
 *
 * This file is part of Commodore 64 emulator.
 * See README for copyright notice
 *
 *
 *
 * Written by
 *   Teemu Rantanen (tvr@cs.hut.fi)
 *
 * $Log:	timer.c,v $
 * Revision 1.3  93/06/21  13:39:37  jopi
 *  X64 version 0.2 PL 0
 * 
 * Revision 1.2  93/06/21  13:09:45  jopi
 * *** empty log message ***
 * 
 * Revision 1.3  1993/06/13  08:39:52  sonninen
 * *** empty log message ***
 *
 * Revision 1.2  1993/06/13  08:21:50  sonninen
 * *** empty log message ***
 *
 *
 */

/*
 * timer.c -- Experimental alarm code. Dynamic approach. Should be fast too.
 */

#include <stdio.h>
#include <stdlib.h>

#include "macro.h"
#include "extern.h"

#ifdef NEW_TIMER_CODE

#define NR_ALARMS 20

CLOCK next_alarm_clk;

typedef struct timer_slot_s
{
    int			  in_use;	/* alarm in use */
    struct timer_slot_s  *next;		/* next alarm to happen */
    CLOCK		  alarm;	/* When ? */
#ifdef __STDC__
    void	        (*func)(int);	/* Call func on alarm */
#else
    void	        (*func)();	/* Call func on alarm */
#endif
    int			  parameter;	/* Passed to callfunc */
} timer_slot_t;


/*
 * Table of alarms
 */
static timer_slot_t alarms[NR_ALARMS];

/*
 * List of alarms to happen
 */
static timer_slot_t *first_alarm = NULL;


#ifdef __STDC__
static void cancel_alarm(timer_slot_t *p);
#else
static void cancel_alarm();
#endif

/*
 * Allocate new alarm.
 */
#ifndef __STDC__
int new_alarm()
#else
int new_alarm(void)
#endif
{
    static int alarms_given = 0;

    if (alarms_given == NR_ALARMS)
    {
	printf("out of alarms\n");
	exit(1);
    }
    return alarms_given++;
}


/*
 * Update alarm clock
 */
#ifndef __STDC__
void find_next_alarm()
#else
void find_next_alarm(void)
#endif
{
    next_alarm_clk = first_alarm ? first_alarm->alarm : (CLOCK) (-1);
}

/*
 * When alarm happens, handle all alarms
 */
#ifndef __STDC__
void alarm_handler()
#else
void alarm_handler(void)
#endif
{
    timer_slot_t *p;
    
    /*
     * Call alarms. Note that if callbackfunction sets alarm <= clk,
     * it will be called again.
     */
    while (first_alarm && clk >= first_alarm->alarm)
    {
	p = first_alarm;
	p->in_use = 0;
	first_alarm = p->next;
	(*(p->func))(p->parameter);
    }

    /*
     * Next alarm
     */
    find_next_alarm();
}

/*
 * Set alarm to happen. Note that if set_alarm is not called from
 * alarm-callback, find_next_alarm must be called.
 */
#ifndef __STDC__
void set_alarm(alarm, value, func, parameter)
int    alarm;
CLOCK  value;
void (*func)();
int    parameter;
#else
void set_alarm(int alarm, CLOCK value, void (*func)(int), int parameter)
#endif
{
    register timer_slot_t *current;
    register timer_slot_t *p;
    register timer_slot_t *prev;

    current = &alarms[alarm];

    /*
     * Remove from list first. This happens only if alarm is active and time
     * is changed
     */
    if (current->in_use)
    {
	if (value == current->alarm)
	    return;
	cancel_alarm(current);
    }

    current->in_use = 1;
    current->alarm = value;
    current->func = func;
    current->parameter = parameter;
    
    prev = NULL;
    p = first_alarm;

    /*
     * If value is same, call alarms in order they are set
     */
    while(p && (value >= p->alarm))
    {
	prev = p;
	p = p->next;
    }

    if (!prev)
    {
	current->next = first_alarm;
	first_alarm = current;
#if 0
	/*
	 * This slows things down 10%. Remember to call find_next_alarm() if
	 * you set alarms outside of alarm callback
	 */
	find_next_alarm();
#endif
	return;
    }

    current->next = p;
    prev->next = current;
}

/*
 * Remove alarm from alarm queue.
 */
#ifndef __STDC__
void clear_alarm(alarm)
int alarm;
#else
void clear_alarm(int alarm)
#endif
{
    register timer_slot_t *p;

    p = &alarms[alarm];

    if (!p->in_use)
	return;

    cancel_alarm(p);
}



/*
 * Remove alarm from list. This will be called quite seldom.
 */
#ifndef __STDC__
static void cancel_alarm(current)
timer_slot_t *current;
#else
static void cancel_alarm(timer_slot_t *current)
#endif
{
    register timer_slot_t *p = first_alarm;
    register timer_slot_t *prev = NULL;

    /*
     * This is the only way...
     */
    while (p)
    {
	if (!p->in_use)
	{
	    printf("alarm list corrupted\n");
	    abort();
	}

	if (p == current)
	{
	    current->in_use = 0;

	    if (prev)
	    {
		prev->next = p->next;
	    }
	    else
	    {
		first_alarm = p->next;
		find_next_alarm();
	    }
	    return;
	}

	prev = p;
	p = p->next;
    }

    printf("cancel_alarm: alarm not on list\n");
    abort();
}

#endif /* NEW_TIMER_CODE */
