/*
 * $Id: vmachine.h,v 1.3 93/06/21 13:39:50 jopi Exp $
 *
 * This file is part of Commodore 64 emulator.
 * See README for copyright notice
 *
 * This file contains useful stuff when you are creating
 * virtual machine like MCS6510 based microcomputer.
 *
 * Included are:
 *	o registers
 *	o memory
 *	o flags in PSW
 *      o addressing modes
 *	o interrupts
 *	o colors
 *
 * Written by
 *   Vesa-Matti Puro (vmp@lut.fi)
 *   Jarkko Sonninen (sonninen@lut.fi)
 *   Jouko Valta (jopi@stekt.oulu.fi)
 *
 * $Log:	vmachine.h,v $
 * Revision 1.3  93/06/21  13:39:50  jopi
 *  X64 version 0.2 PL 0
 * 
 * Revision 1.2  93/06/21  13:09:56  jopi
 * *** empty log message ***
 * 
 * Revision 1.3  1993/06/13  08:39:52  sonninen
 * *** empty log message ***
 *
 * Revision 1.2  1993/06/13  08:21:50  sonninen
 * *** empty log message ***
 *
 *
 */

#ifndef _vmachine_h_included
#define _vmachine_h_included

#include "config.h"

/* 6510 registers. */

#define AC		accumulator
#define XR		x_register
#define YR		y_register
#define SP		stack_pointer
#define PC		program_counter
#define PCH		((PC>>8)&0xff)
#define PCL		(PC&0xff)

#define ZF		zero_flag
#define SF		sign_flag
#define OF		overflow_flag
#define BF		break_flag
#define DF		decimal_flag
#define IF		interrupt_flag
#define CF		carry_flag

#define BYTE 		unsigned char
#define ADDRESS 	unsigned int
#define CLOCK 		unsigned long

#define RAMSIZE		0x10000
#define ROMSIZE		0x2000
#define EXROMSIZE	0x2000  /* module is 2*8 KB slots */

/* Masks which indicate location of status flags in PSW. */
#define SIGN		0x80
#define OVERFLOW	0x40
#define NOTUSED 	0x20
#define BREAK	    	0x10
#define DECIMAL		0x08
#define INTERRUPT	0x04
#define ZERO		0x02
#define CARRY		0x01


/* ADDRESSING MODES */
#define IMPLIED      0

#define IMMEDIATE    1

#define ZERO_PAGE    2
#define ZERO_PAGE_X  3
#define ZERO_PAGE_Y  4

#define ABSOLUTE     5
#define ABSOLUTE_X   6
#define ABSOLUTE_Y   7

#define INDIRECT     8
#define INDIRECT_X   9
#define INDIRECT_Y   10

#define RELATIVE     11

#define ACCUMULATOR  12

#define ASS_CODE     13


/* INRTERRUPS */

#define I_BRK		0	/* software interrupt */
/* VIC IRQ */
#define I_LIGHT_PEN	1
#define I_SS_COL	2	/* sprite to sprite collision */
#define I_SB_COL	3	/* sprite to background collision */
#define I_RASTER	4	/* raster compare */

/* CIA 1 IRQs */
#define I_CIA1FL	5	/* CIA 1 FLAG1 */
#define I_CIA1SP	6	/* CIA 1 serial port */
#define I_CIA1TOD	7	/* CIA 1 TOD alarm */
#define I_CIA1TA	8	/* CIA 1 timer A */
#define I_CIA1TB	9	/* CIA 1 timer B */

/* CIA 2 NMIs */
#define I_CIA2FL	10	/* CIA 2 FLAG 1 */
#define I_CIA2SP	11	/* CIA 2 serial port */
#define I_CIA2TOD	12	/* CIA 2 TOD alarm */
#define I_CIA2TA	13	/* CIA 2 timer A */
#define I_CIA2TB	14	/* CIA 2 timer B */

#define I_SPECX		15	/* special "interrupt" for x_loop */
#define I_RESTORE	16	/* restore key NMI */
#define I_RESET		17	/* reset in startup */
#define NUMOFINT	18

/* border, background & sprite colors */
#define C_BORDER	0
#define C_BG0		1
#define C_BG1		2
#define C_BG2		3
#define C_BG3		4
#define C_SPRMUL0	5
#define C_SPRMUL1	6
#define C_SPR0		7
#define C_SPR1		8
#define C_SPR2		9
#define C_SPR3		10
#define C_SPR4		11
#define C_SPR5		12
#define C_SPR6		13
#define C_SPR7		14

/* display modes */
#define V_BLANK		0
#define V_TEXT		1
#define V_MTEXT		2
#define V_ECTEXT	3
#define V_BITMAP	4
#define V_MBITMAP	5
#define V_TILT		6

/* C64 constants */

/* video constants */
/*
 * exact values are more complicated, but these approximations are
 *  enough for emaltor's use
 */


#ifdef PAL
#define CYCLES_PER_SEC 985248
#define RFSH_PER_SEC 50
#define SCR_LINES 312
#else

#define CYCLES_PER_SEC 1022730
#define RFSH_PER_SEC 60
#define SCR_LINES 255
#endif

#define CYCLES_PER_RFSH (CYCLES_PER_SEC/RFSH_PER_SEC)
#define CYCLES_PER_LINE (CYCLES_PER_RFSH/SCR_LINES)
#define CURRENT_LINE(clk) (((clk)/CYCLES_PER_LINE) % SCR_LINES)


/*
 * This file contains declaration for lookup-table which is
 * used to translate MOS6502 machine instructions. Machine code
 * is used as index to array called lookup. Pointer to function
 * is then fetched from array and function is called.
 */

extern struct lookup_tag {

#ifdef HACK
    void    (*funktio) ();	/* "Funktio" is Finnish and means function. */
#else
    unsigned int (*funktio) ();	/* "Funktio" is Finnish and means function. */
#endif
    int     addr_mode;	/* Selfdocumenting? */
    int     source;
    int     destination;
    char   *mnemonic;	/* Selfdocumenting? */
    int     cycles;
}       lookup[];


/* Addressing mode (addr_mode) is used when instruction is diassembled
 * or assembled by diassembler or assembler. This is used i.e.
 * in function char *sprint_opcode() in the file misc.c.
 *
 * MOS6502 addressing modes are #defined in the file "vmachine.h".
 *
 * Mnemonic is character string telling the name of the instruction.
 */

#define M_NONE	0
#define M_AC 	1
#define M_XR	2
#define M_YR	3
#define M_SP	4
#define M_SR	5
#define M_PC	6
#define M_IMM	7
#define M_ZERO	8
#define M_ZERX	9
#define M_ZERY	10
#define M_ABS	11
#define M_ABSX	12
#define M_ABSY	13
#define M_IND	14
#define M_INDX	15
#define M_INDY	16
#define M_REL	17
#define M_FC	18
#define M_FD	19
#define M_FI	20
#define M_FV	21
#define M_ADDR	22
#define M_	23

#ifndef NO_UNDOC_CMDS
#define M_ACIM	24	/* Source: AC & IMMED (bus collision) */
#define M_ANXR	25	/* Source: AC & XR (bus collision) */
#define M_AXIM	26	/* Source: (AC | #EE) & XR & IMMED (bus collision) */
#define M_ACNC	27	/* Dest: M_AC and Carry = Negative */
#define M_ACXR	28	/* Dest: M_AC, M_XR */

#define M_SABY	29	/* Source: (ABS_Y & SP) (bus collision) */
#define M_ACXS	30	/* Dest: M_AC, M_XR, M_SP */
#define M_STH0	31	/* Dest: Store (src & Addr_Hi+1) to (Addr +0x100) */
#define M_STH1	32
#define M_STH2	33
#define M_STH3	34

#else
#define M_ACIM	M_NONE
#define M_ANXR	M_NONE
#define M_AXIM	M_NONE
#define M_ACNC	M_NONE
#define M_ACXR	M_NONE

#define M_SABY	M_NONE
#define M_ACXS	M_NONE
#define M_STH0	M_NONE
#define M_STH1	M_NONE
#define M_STH2	M_NONE
#define M_STH3	M_NONE
#endif


/*
 * Trap structure. 3 bytes are stored for simple checking that correct ROM
 * is installed.
 */
typedef struct trap_s
{
    char 	*name;
    ADDRESS 	 address;
    BYTE	 check[3];
#ifdef __STDC__
    void       (*func)(void);
#else
    void       (*func)();
#endif
} trap_t;

#endif
